#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces.contact import IContactParagraph
from pyams_content.component.paragraph.interfaces.frame import IFrameParagraph
from pyams_content.component.paragraph.interfaces.keypoint import IKeypointsParagraph
from pyams_content.component.paragraph.interfaces.verbatim import IVerbatimParagraph
from pyams_content.component.paragraph.interfaces.video import IVideoParagraph
from pyams_content.component.video.interfaces import IExternalVideoParagraph
from pyams_i18n.interfaces import II18nManager
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.gallery.interfaces import IGalleryParagraph
from pyams_content.component.illustration.interfaces import IIllustrationParagraph
from pyams_content.component.paragraph.interfaces import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph, IRawParagraph
from pyams_content.shared.imagemap.interfaces import IImageMapParagraph
from pyams_content_es.interfaces import IDocumentIndexInfo

# import packages
from pyams_utils.adapter import adapter_config
from pyams_utils.html import html_to_text


@adapter_config(name='body', context=IParagraphContainerTarget, provides=IDocumentIndexInfo)
def ParagraphContainerTargetIndexInfo(content):
    """Paragraph container index info"""
    body = {}
    for paragraph in IParagraphContainer(content).values():
        if not paragraph.visible:
            continue
        info = IDocumentIndexInfo(paragraph, None)
        if info is not None:
            for lang, info_body in info.items():
                body[lang] = '{old}\n{body}'.format(old=body.get(lang, ''),
                                                    body=info_body)
    return {'body': body}


@adapter_config(context=IBaseParagraph, provides=IDocumentIndexInfo)
def BaseParagraphIndexInfo(paragraph):
    """Base paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    return info


@adapter_config(context=IHeaderParagraph, provides=IDocumentIndexInfo)
def HeaderParagraphIndexInfo(paragraph):
    """Header paragraph index info"""
    info = {}
    for lang, header in (getattr(paragraph, 'header', {}) or {}).items():
        if header:
            info.setdefault(lang, header)
    return info


@adapter_config(context=IHTMLParagraph, provides=IDocumentIndexInfo)
def HTMLParagraphIndexInfo(paragraph):
    """Rich text paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    for lang, body in (getattr(paragraph, 'body', {}) or {}).items():
        if body:
            new_body = '{old}\n{body}'.format(old=info.get(lang, ''),
                                              body=html_to_text(body).replace('\r', ''))
            info[lang] = new_body
    return info


@adapter_config(context=IIllustrationParagraph, provides=IDocumentIndexInfo)
def IllustrationIndexInfo(paragraph):
    """Illustration index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    for lang, legend in (getattr(paragraph, 'alt_title', {}) or {}).items():
        if legend:
            new_legend = '{old}\n{legend}'.format(old=info.get(lang, ''),
                                                  legend=legend)
            info[lang] = new_legend
    return info


@adapter_config(context=IGalleryParagraph, provides=IDocumentIndexInfo)
def GalleryParagraphIndexInfo(paragraph):
    """Gallery paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    for image in IGalleryParagraph(paragraph).values():
        for attr in ('title', 'alt_title', 'description', 'author_comments', 'sound_title', 'sound_description'):
            for lang, value in (getattr(image, attr, {}) or {}).items():
                if value:
                    new_info = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                       value=value or '')
                    info[lang] = new_info
    return info


@adapter_config(context=IVideoParagraph, provides=IDocumentIndexInfo)
def VideoParagraphIndexInfo(paragraph):
    """Video paragraph index info"""
    info = {}
    manager = get_parent(paragraph, II18nManager)
    for lang in manager.get_languages():
        body = (paragraph.body or {}).get(lang, '')
        if body:
            info[lang] = '{old}\n{body}'.format(old=info.get(lang, ''),
                                                body=html_to_text(body.replace('\r', '')))
        for attr in ('title', 'description'):
            value = getattr(paragraph, attr, {}).get(lang, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
        for attr in ('author', ):
            value = getattr(paragraph, attr, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
    return info


@adapter_config(context=IExternalVideoParagraph, provides=IDocumentIndexInfo)
def ExternalVideoParagraphIndexInfo(paragraph):
    """External video paragraph index info"""
    info = {}
    manager = get_parent(paragraph, II18nManager)
    for lang in manager.get_languages():
        body = (paragraph.body or {}).get(lang, '')
        if body:
            info[lang] = '{old}\n{body}'.format(old=info.get(lang, ''),
                                                body=html_to_text(body.replace('\r', '')))
        for attr in ('title', 'description'):
            value = getattr(paragraph, attr, {}).get(lang, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
        for attr in ('author',):
            value = getattr(paragraph, attr, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
    return info


@adapter_config(context=IImageMapParagraph, provides=IDocumentIndexInfo)
def ImagemapParagraphIndexInfo(paragraph):
    """Image map paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    return info


@adapter_config(context=IContactParagraph, provides=IDocumentIndexInfo)
def ContactParagraphIndexInfo(paragraph):
    """Contact paragraph index info"""
    info = {}
    manager = get_parent(paragraph, II18nManager)
    for lang in manager.get_languages():
        for attr in ('title', 'charge'):
            value = getattr(paragraph, attr, {}).get(lang, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
        for attr in ('name', 'address'):
            value = getattr(paragraph, attr, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
    return info


@adapter_config(context=IKeypointsParagraph, provides=IDocumentIndexInfo)
def KeypointsParagraphIndexInfo(paragraph):
    """Key points paragraph index info"""
    info = {}
    for lang, body in (getattr(paragraph, 'body', {}) or {}).items():
        if body:
            info.setdefault(lang, body)
    return info


@adapter_config(context=IFrameParagraph, provides=IDocumentIndexInfo)
def FrameParagraphIndexInfo(paragraph):
    """Frame paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    for lang, body in (getattr(paragraph, 'body', {}) or {}).items():
        if body:
            info[lang] = '{old}\n{body}'.format(old=info.get(lang, ''),
                                                body=html_to_text(body.repplace('\r', '')))
    return info


@adapter_config(context=IVerbatimParagraph, provides=IDocumentIndexInfo)
def VerbatimParagraphIndexInfo(paragraph):
    """Verbatim paragraph index info"""
    info = {}
    manager = get_parent(paragraph, II18nManager)
    for lang in manager.get_languages():
        for attr in ('title', 'quote', 'charge'):
            value = getattr(paragraph, attr, {}).get(lang, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
        for attr in ('author', ):
            value = getattr(paragraph, attr, '')
            if value:
                info[lang] = '{old}\n{value}'.format(old=info.get(lang, ''),
                                                     value=value)
    return info


@adapter_config(context=IRawParagraph, provides=IDocumentIndexInfo)
def RawParagraphIndexInfo(paragraph):
    """Raw paragraph index info"""
    info = {}
    for lang, title in (getattr(paragraph, 'title', {}) or {}).items():
        if title:
            info.setdefault(lang, title)
    for lang, body in (getattr(paragraph, 'body', {}) or {}).items():
        if body:
            new_body = '{old}\n{body}'.format(old=info.get(lang, ''),
                                              body=html_to_text(body).replace('\r', ''))
            info[lang] = new_body
    return info
